% Transport, reaction and boundary condition vectors are automatically 
% generated via autoMEDIALAB.m in form of MATLAB function handles and are 
% passed to mainMEDIALAB.m. 

function  [speciesPhase, stochiometrix, advection, diffusion, reaction, plBC, prBC] = autoMEDIALAB(speciesName,inputFile, r, R, reactionUnits)

% This function :
%   1) Reads the model parameters from inputMEDIALAB.txt
%   2) constructs reaction network stochiometrics matrix as well as reaction
%      transport and upper boundary condition components vector functions 
%      in the form of MATLAB function handles.
%
% output arguments:
%   speciesPhase        - vector of species phases i.e. 1 for solids 
%                         and 0 for solutes. It has the size of numSpecies
%   stochiometrix       - metrice of stochimetrics coefficients. It has
%                         the size of [numSpecies*numRxns] where numRxns is
%                         the number of reactions in the reaction network
%   advection           - vector of function handle for the advection term
%                         of species. It has the size of [numSpecies*1]
%   diffusion           - vector of function handle for the diffusion
%                         of species. It has the size of [numSpecies*1] 
%   reaction            - vector of function handle for reaction rates.
%                         It has the size of [numSpecies*1]
%   plBC                - vector of function handle of the upper
%                         boundary condition.It has the size of 
%                         [numSpecies*1]
%   prBC                - vector of function handle of the lower
%                         boundary condition.It has the size of 
%                         [numSpecies*1]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%******************************1st BLOCK**********************************%
%************************** READING INPUT DATA ***************************%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('**********************');
disp('READING THE INPUT FILE');

 fid            = fopen(inputFile,'r');                                    % reading the parameters from input file
 C              = textscan(fid,'%s%s');                                    % %s ist ein Formatierungs Befehl
 rateConsNames  = C{1};                                                    % extracting the names of parameters
 rateConsVals   = C{2};                                                    % extracting the values of parameters
 fclose(fid);                                                              % close file

 for i=1:length(rateConsVals)
     disp(strcat(rateConsNames(i), ' = ',rateConsVals(i)));                % printing on the screen names and vlaues of parameters
 end
disp('**********************');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%******************************2nd BLOCK**********************************%
%********************** EXTRACTING PHASES OF SPECIES *********************%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ruft species userMediaLab auf zhlt sie und checkt ob species aq oder s und wandelt
% dies in 0 bzw 1 um
numSpecies = int16(length(speciesName)) ;
for i=1:length(speciesName)
    if (isempty(strfind(speciesName{i},'(aq)'))); 
        speciesPhase(i) = 1;                                               % speciesPhase equals to 1 if species is a solid
        speciesName{i} = strrep(speciesName{i},'(s)','');
    else
        speciesPhase(i)=0;                                                 % speciesPhase equals to 0 if species is a solute
        speciesName{i} = strrep(speciesName{i},'(aq)','');
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%******************************3rd BLOCK**********************************%
%**************** CALCULATING REACTION NETWORK STOCHIMETICS **************%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('CALCULATING REACTION NETWORK STOCHIOMETRICS'); 
 for i=1:numSpecies
    for j=1:length(R)
         c=coeffs(R{j},speciesName{i});                                    % extracting coeffiecints of each reaction
         if (length(c)>1)
             if (((speciesPhase(i)==1)&&(reactionUnits(j)==1))||((speciesPhase(i)==0)&&(reactionUnits(j)==0)))
                stochiometrix(i,j) = -c(2);                                % if both species and reaction have same units, the coefficient wouldn't be corrected by conversion factor otherwise conFactor will be imposed as follows
             elseif ((speciesPhase(i)==1)&&(reactionUnits(j)==0))
                stochiometrix(i,j) = -c(2)/str2num(rateConsVals{find(ismember(rateConsNames,'convFactor'))});
             else 
                 stochiometrix(i,j) = -c(2)*str2num(rateConsVals{find(ismember(rateConsNames,'convFactor'))});
             end;      
         else
             stochiometrix(i,j) = 0;                                       % if species is not incorporated in the reaction, then the coefficient equals to zero
         end

     end
 end
 disp(num2str(double(stochiometrix)));
 disp('**********************');
 % Hier wird der Symbolik mit den Coeffizenten verbunden und die
 % Substansnamen eingefgt
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%******************************4th BLOCK**********************************%
%************** CONSTRUCTING FUNCTION HANDLE OF REACTION RATES ***********%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('CONSTRUCTING FUNCTION HANDLE OF REACTION RATES');
for nRxn=1:length(R)
     for nSpec=1:numSpecies
         uIndex     = strcat('u(',strcat(num2str(nSpec),')'));
         rnew{nRxn} = strrep(r{nRxn}, speciesName{nSpec}, uIndex);         % reaction rate terms 'r' are replaced by their numeric values u(i) where i is the index of the species in the sequential vector of diagenesis equations
         r{nRxn}    = rnew{nRxn};
     end
     for nRateCons=1:length(rateConsVals)
         rnew{nRxn} = strrep(r{nRxn}, rateConsNames{nRateCons}, char(rateConsVals(nRateCons)));
         r{nRxn}    = rnew{nRxn};
     end
        r{nRxn} = strcat('@(u)',r{nRxn});
        r{nRxn} = str2func(r{nRxn});
end
disp(' FUNCTION HANDLE OF REACTION RATES = '); 
r
disp('**********************');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%******************************5th BLOCK**********************************%
% CONSTRUCTING FUNCTION HANDLE OF ADVECTION, DIFFUSION AND BOUNDARY CONDITIONS%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('CONSTRUCTING FUNCTION HANDLE OF ADVECTION, DIFFUSION AND BOUNDARY CONDITION');

for i=1:numSpecies
     advString {i} = strcat('@(u)-w*u(',num2str(i),')');
     prString{i}   = strcat('@(ur)',char(rateConsVals{find(ismember(rateConsNames,'w'))}),'*ur(',num2str(i),')*porosity');
     if (speciesPhase(i)==0)
        D             = strcat('+Dmol_',speciesName{i},'/',num2str(1/(1-log(str2num(rateConsVals{find(ismember(rateConsNames,'porosity'))}))^2)));
        plString{i}   = strcat('@(ul,t)ul(',num2str(i),')-BC_',speciesName{i});
        difString {i} = strcat('@(DuDx)(D_bio',D,')*DuDx(',num2str(i),')');
     else
        plString{i}   = strcat('@(ul,t)BC_',speciesName{i},'/convFactor/',char(rateConsVals{find(ismember(rateConsNames,'porosity'))}));
        difString {i} = strcat('@(DuDx)D_bio*DuDx(',num2str(i),')');
     end
     for nRateCons=1:length(rateConsVals)
         advStringNew{i} = strrep(advString{i}, rateConsNames{nRateCons}, char(rateConsVals(nRateCons)));
         advString{i}    = advStringNew{i};
         difStringNew{i} = strrep(difString{i}, rateConsNames{nRateCons}, char(rateConsVals(nRateCons)));
         difString{i}    = difStringNew{i};
         plStringNew{i}  = strrep(plString{i}, rateConsNames{nRateCons}, char(rateConsVals(nRateCons)));
         plString{i}     = plStringNew{i};
         prStringNew{i}  = strrep(prString{i}, rateConsNames{nRateCons}, char(rateConsVals(nRateCons)));
         prString{i}     = prStringNew{i};
     end
         advHandle{i}    = str2func(advString{i});
         difHandle{i}    = str2func(difString{i});
         plHandle{i}     = str2func(plString{i});
         prHandle{i}     = str2func(prString{i});
end


disp(' FUNCTION HANDLE OF ADVECTION = '); 
advHandle
disp('**********************');

disp(' FUNCTION HANDLE OF DIFFUSION = ');
difHandle
disp('**********************');

disp(' FUNCTION HANDLE OF UPPER BOUNDARY CONDITION = '); 
plHandle
disp('**********************');

disp(' FUNCTION HANDLE OF LOWER BOUNDARY CONDITION = '); 
prHandle
disp('**********************');

%*************************************************************************%
  stochiometrix = double (stochiometrix);
  reaction  = r;
  advection = advHandle;
  diffusion = difHandle;
  plBC      = plHandle;
  prBC      = prHandle;
 
             
             
        