%**************************************************************************
%**************************************************************************
%           MEDIALAB (Modeling Early DIAgenesis using MATLAB) v.(1.0)
%                               March 2014
% MEDIALAB is an early diagenesis model, which calculates concentrations,
% fluxes of chemical species and rates of all the biogeochemical pathways 
% at each depth of aquatic sediments for a specific time period. 
% A system of partial differential equations corresponding to early
% diageneis equation are automatically generated through MATLABs symbolic
% programming capabilities and solved using MATLABs built-in solver pdepe 
% to evaluate temporal and spatial distribution of chemical species.
% MEDIALAB is executed through MATLAB home screen through following command
%
%     [simValues, depth, time] = mainMEDIALAB
%
% Execution of MEDIALAB requires an active installation of MATLAB 
% (Version 7.6 release R2008a or later ). It is recommended to allow at 
% least 1GB of space on the hard drive for the model output and 1GB of 
% contiguous random-access memory for the initialization routine.
%**************************************************************************
%**************************************************************************

function [simValues, depth, time] = mainMEDIALAB
% mainMEDIALAB is the core script file of MEDIALAB which is executed from 
% MATLABs home screen. Home directory must be the same address as the 
% folder containing the contents of MEDIALAB files unless it has been saved 
% in the similar directory. It is in this script that userMEDIALAB.m,
% autoMEDIALAB and pdepe solvers are called and the final solution, sol, is
% saved in the resultMEDIALAB.mat which involves all the simulated 
% concentrations, depth and time values in a MATLAB matrix format. After 
% termination of simulation, variables of depth, time and simValues are 
% saved in the workspace. They are used for plotting and reaction rate 
% calculations through postprocessMEDIALAB.m.

% This function solves PDEs of early diageneis in lake sediments (here,
% Lake Baldegg in Switzerland)
% It uses 'pdepe' solver of MATLAB which solves 1-D elliptic PDEs
%
%
% output arguments:
%   simValues          - stores the simulated concentrations of all the 
%                        species at all depths and times. Its size is
%                        {numSpecies*1} cells where each cell stores
%                        concentrations of the species at nX depths and nT 
%                        times i.e. [nT*nX] metrices.
%   depth              - the spatial interval in [cm] with nX descritized 
%                        depths
%   time               - the temporal interval in [yr] with nT descritized
%                        times



tic;                                                                       % starts stopwatch timer                                                               % clears all the varibles in the workspace memory
close all;                                                                 % closes all the matlab figures

% calling function userMEDIALAB which has been provided by the user and
% contains spatial and temporal domains and reaction network
%
% output arguments:
%   x                   - spatial interval i.e. [0,L] with nX
%                         descritized depths
%   t                   - temporal domain i.e. [0,T] with nT decritized
%                         times
%   speciesName         - cell array containing strings of names of
%                         species. Name of species end with (s) or (aq)
%                         as a flag variable to distinuish between solid or
%                         aqeouse species respectively.
%   inputFile           - string variable of name of the input file. It is
%                         a text file and has extention of .txt
%   r                   - cell array with elements of reactions kinetic
%                         rates expresses in a string format. It has the
%                         sieze of {numRxns*1} where numRxns is number of 
%                         reactions
%   R                   - cell vector with elements of symbolic terms of
%                         reactions. Its size is {numRxns*1} 
%   reactionUnits       - vector containg 0 or 1 values when reactions have 
%                         solute or solid species units.
[x, t, speciesName, inputFile, r, R, reactionUnits] = userMEDIALAB;  

% calling function autoMEDIALAB which automatically calculates components
% of pdepe solver i.e. vector functions of transport, reaction and boundary 
% conditions using symbolic programming capability of MATLAB
%
% output arguments:
%   speciesPhase        - vector of species phases i.e. 1 for solids 
%                         and 0 for solutes. It has the size of numSpecies
%   stochiometrix       - metrice of stochimetrics coefficients. It has
%                         the size of [numSpecies*numRxns] where numRxns is
%                         the number of reactions in the reaction network
%   advection           - vector of function handle for the advection
%                         of species. It has the size of [numSpecies*1]
%   diffusion           - vector of function handle for the diffusion
%                         of species. It has the size of [numSpecies*1] 
%   reaction            - vector of function handle for reaction rates.
%                         It has the size of [1*numRxns]
%   plBC                - vector of function handle of the upper
%                         boundary condition.It has the size of 
%                         [numSpecies*1]
%   prBC                - vector of function handle of the lower
%                         boundary condition.It has the size of 
%                         [numSpecies*1]
[speciesPhase, stochiometrix, advection, diffusion, reaction, plBC, prBC] = autoMEDIALAB(speciesName,inputFile, r, R, reactionUnits);


numSpecies = length(speciesName);                           
depth = x;
time  = t;


%display starting time of solving PDE
rt_time = clock;                                    
fprintf('\nThe starting time is %.0f/%.0f/%.0f %.0f:%.0f:%.0f\n', rt_time(1),...
        rt_time(2), rt_time(3), rt_time(4), rt_time(5), rt_time(6));
disp('solving PDE ');

% calls the pdepe solver and stores the result in 'sol' which is
% [nT*nX*numSpecies] metrice. 3 functions of pdeFun, icFun and bcFun are
% called automatically by pdepe. 
m = 0;                                                                     % m value depends on the symmetry of the problem and for 1D early diagenesis problem it equals to zeo 
sol = pdepe(m,@pdeFun,@icFun,@bcFun,x,t);    
toc;                                     
%display ending time of simulation
rt_time = clock;                                  
fprintf('The ending time is %.0f/%.0f/%.0f %.0f:%.0f:%.0f\n', rt_time(1),...
         rt_time(2), rt_time(3), rt_time(4), rt_time(5), rt_time(6));

simValues = cell(numSpecies, 1);                                           %creates an numSpecies*1 cell array of empty matrices for the concentration
for j = 1:numSpecies
    simValues{j} = sol(:,:,j);                      
end                                                 

save('resultMEDIALAB.mat','simValues','sol');                              %saves simulation results in resultMEDIALAB matrix

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% pdeFun constructs c,f,s that are column vectors of time gradient
% coefficients, transport and reaction
%********************* TRANSPORT AND REACTION ****************************%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [c,f,s] = pdeFun(x,t,u,DuDx)
t
% coupling of the partial derivatives with respect to time is restricted to
% multiplication by a diagonal matrix c (x,t,u,?u/?x). In no-adsorption
% diagenesis problem, it equals to one
c = ones(numSpecies,1);

%Transport vecotr function
for i=1: length(advection) 
    valAdvection (i) = advection{i}(u);
    valDiffusion (i) = diffusion{i}(DuDx);
end
f = valAdvection + valDiffusion;
f=f';
%Reaction vector function
for i=1: length(reaction) 
    valReaction(i) = reaction{i}(u);
end
s = real(stochiometrix*(valReaction'));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  calculates initial condition vector function for pdepe solver
%************************** INITIAL CONDITION ****************************%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function u0 = icFun(x)                            
u0 = zeros(numSpecies,1);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% bcFun calculates boundary condition vector function at lower and upper
% boundaries
%************************* BOUNDARY CONDITION ****************************%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [pl,ql,pr,qr] = bcFun(xl,ul,xr,ur,t)   
%Upper bounday 

for i=1:length(plBC)
    pl(i) = plBC{i}(ul,t);
end
ql = speciesPhase;
ql=ql'; %gendert von ql'
pl=pl'; %gendert von pl'
%Lower bounday 
for i=1:length(prBC)
    pr(i) = prBC{i}(ur);
end
pr=pr'; %gendert von pr'

qr = ones(numSpecies,1);
%*************************************************************************%
end
end
