% userMEDIALAB is the first script that has to be modified by the user to
% provide all the necessary information of spatial and temporal domains as 
% well as biogeochemical reaction network. There are some other parameters
% such as list of species names, organic matter composition coefficients 
% and input file name that have to be included in the script. Species are 
% characterized by string variables and their assigned names contain either
% (s) or (aq) representing solid or aqueous species, respectively. This
% is an approach to characterize phase of the species in the script when 
% building the transport matrix (as molecular diffusion is excluded in case
% of solid species) and reaction matrix ( to ensure consistency between 
% reactions and species units). As described above, variable names used for 
% the chemical species are defined as symbols to set up the reaction 
% network. 

function  [x, t, speciesName, inputFile, r, R, reactionUnits] = userMEDIALAB  

% In this function which must be edited by the user:
%  1) sptial and temporal domains of x and t are defined
%  2) chemical composition of OM i.e carbon (Cx), nitrogen(Cy) and 
%     phosphorous (Cz) content of OM is defined. In the current model there
%     are 3 pools of OM which represent labile, less labile and refractory 
%     fractions
%  3) text file, inputFile.txt, containing the input parameters is provided
%  4) name of the species that diagensis equation is solved for
%  5) user defines the reactions, R, in algebraic form applying 
%     MATLAB's symbolic programming capabilitieseach and their 
%     corresponding rates, r, in a string format . For reaction defined  
%     by R{i} in this script there are two attributes: reaction rates 
%     defined by r{i} and reactionUnits(i). 

%************************** USER DEFINED *********************************%
% definining the spatial resolution and total sediment depth to be modeled.
% x = linspace(0, L, L/0.1) creats a domain of L/0.1 depths in the spatial 
% interval of [0,L]. The depth resolution of 0.1cm is an adequate precision 
% for most of the diagenesis problem. Units are in [cm]. Here 20 cm of
% sediment is simulated
x = linspace(0,20,200); 
%
% defining time span of the simulation and the temporal resolution.
% t = linspace(0, T, nT) cteats a simuation period of T years with nT
% descritized points. Units are in [yr].
   t = linspace(0,100,400);
% carbon content of 1st OM pool
cx1 = 106;
% nitrogen content of 1st OM pool
cy1 = 16;
% phosphorous content of 1st OM pool
cz1 = 1;
% carbon content of 2nd OM pool
cx2 = 106;
% nitrogen content of 2nd OM pool
cy2 = 16;
% phosphorous content of 2nd OM pool
cz2 = 1;
% input file for the model's parameters
inputFile = 'inputMEDIALAB.txt';
% species names including their phase i.e. solid or solute which is denoted
% by (s) or (aq)
speciesName = {'o2(aq)',...      % oxygen  
               'no3(aq)',...     % nitrate
               'mo_1(s)',...     % 1st manganese oxide pool
               'foh(s)',...      % iron hydroxide pool
               'so4(aq)',...     % sulfate
               'nh4(aq)',...     % ammonium
               'mn(aq)',...      % manganese(II) 
               'fe(aq)',...      % iron(II)
               's2(aq)',...      % sulfide(II)
               'fs(s)',...       % iron sulfide (FeS)
               'pyrite(s)',...   % pyrite
               'om1(s)',...      % 1st organic matter pool
               'om2(s)', ...     % 2nd organic matter pool
               'om3(s)',...      % 3rd organic matter pool
               'hpo4(aq)',...    % phosphate
               'ch4(aq)',...     % methane
               'mo_2(s)',...     % 2nd manganese oxide
               'vivianite(s)'};  % vivianite Fe3(PO4)2

         
% all of the species included in the reactions must be included in this
% list
syms om1 om2 om3 mo_1 mo_2 foh fs pyrite vivianite...
     o2 no3 so4 nh4 mn fe s2 ch4 hpo4 hco_3 co_2 h2o n2 h h2
        

 % R{i}            - the cell array of reactions. i=1:numRxns where numRxns 
 %                   are the total number of reactions in the network. 
 %                   For an irreversible reaction      aA + bB ---> cC + dD 
 %                   the R{i} written is 
 %                                                    a*A + b*B - c*C - d*D
 %                   For a reversible reaction         
 %                                                    aA + bB <---> cC + dD 
 %                   the R{i} will be
 %                                                    a*A + b*B - c*C - d*D  
 %                   and the R{i+1} will be
 %                                                    c*C + d*D - a*A - b*B
 % r{i}            - the cell array of reaction rates associated with each
 %                   reaction. It is a string variable
 %
 % reactionUnits(i)- defines whether the reaction has the unit of a solid
 %                   i.e. umol/gr which equals to 1 or a solute with a unit 
 %                   of umol/cm3 which will equal to 0
 %
 %
 %
 % om1 oxidation by o2
 R{1}             = cx1*om1 + cx1*o2 + (-cy1+2*cz1)*hco_3 -(cx1+cy1+2*cz1)*co_2 - (cx1+2*cy1+2*cz1)*h2o - cz1*hpo4 - cy1*nh4;
 r{1}             = 'k1_OM * om1 * o2/(o2 + kO) / cx1';
 reactionUnits(1) = 1;
 
 % om2 oxidation by o2
 R{2}             = cx2*om2 + cx2*o2 + (-cy2+2*cz2)*hco_3 -(cx2-cy2+2*cz2)*co_2 - (cx2+2*cy2+2*cz2)*h2o - cz2*hpo4 - cy2*nh4;
 r{2}             = 'k2_OM * om2 * o2/(o2 + kO) / cx2';
 reactionUnits(2) = 1;

 % om1 denitrification 
 R{3}             = cx1*om1 + 0.8*cx1*no3 - (0.8*cx1+cy1-2*cz1)*hco_3 -(0.2*cx1-cy1+2*cz1)*co_2 - (0.6*cx1-cy1+2*cz1)*h2o - 0.4*cx1*n2 - cz1*hpo4 - cy1*nh4;
 r{3}             = 'k1_OM * om1 * no3/(no3 + kNO) * kinO/(kinO + o2) / cx1 ';
 reactionUnits(3) = 1;
 
 % om2 denitrification 
 R{4}             = cx2*om2 + 0.8*cx2*no3 - (0.8*cx2+cy2-2*cz2)*hco_3 -(0.2*cx2-cy2+2*cz2)*co_2 - (0.6*cx2-cy2+2*cz2)*h2o - 0.4*cx2*n2 - cz2*hpo4 - cy2*nh4;
 r{4}             = 'k2_OM * om2 * no3/(no3 + kNO) / cx2 ';
 reactionUnits(4) = 1;

 % om1 oxidation by 1st MnO2 pool (mo_1)
 R{5}       	  = cx1*om1 + 2*cx1*mo_1 + (3*cx1+cy1-2*cz1)*co_2 + (cx1+cy1-2*cz1)*h2o - (4*cx1+cy1-2*cz1)*hco_3 - 2*cx1*mn - cz1*hpo4 - cy1*nh4;
 r{5}             = 'k1_OM * om1 * mo_1/(mo_1 + kMO) * kinO/(kinO + o2) * kinNO/(kinNO + no3) / cx1';
 reactionUnits(5) = 1;
 
 % om2 oxidation by 1st MnO2 pool (mo_1)
 R{6}             = cx2*om2 + 2*cx2*mo_1 + (3*cx2+cy2-2*cz2)*co_2 + (cx2+cy2-2*cz2)*h2o - (4*cx2+cy2-2*cz2)*hco_3 - 2*cx2*mn - cz2*hpo4 - cy2*nh4;
 r{6}             = 'k2_OM * om2 * mo_1/(mo_1 + kMO) * kinO/(kinO + o2) * kinNO/(kinNO + no3) / cx2';
 reactionUnits(6) = 1;
 
 % om1 oxidation by Fe(OH)  (foh)
 R{7}             = cx1*om1 + 4*cx1*foh + (7*cx1+cy1-2*cz1)*co_2 - (3*cx1+cy1-2*cz1)*h2o - (8*cx1+cy1-2*cz1)*hco_3 - 4*cx1*fe - cz1*hpo4 - cy1*nh4;
 r{7}             = 'k1_OM * om1 * foh/(foh + kFOH) * kinO/(kinO + o2) * kinNO/(kinNO + no3) * kinMO/(kinMO + mo_1)/ cx1 ';
 reactionUnits(7) = 1;
 
 % om2 oxidation by Fe(OH)  (foh)
 R{8}             = cx2*om2 + 4*cx2*foh + (7*cx2+cy2-2*cz2)*co_2 - (3*cx2+cy2-2*cz2)*h2o - (8*cx2+cy2-2*cz2)*hco_3 - 4*cx2*fe - cz2*hpo4 - cy2*nh4;
 r{8}             = 'k2_OM * om2 * foh/(foh + kFOH) * kinO/(kinO + o2) * kinNO/(kinNO + no3) * kinMO/(kinMO + mo_1) / cx2';
 reactionUnits(8) = 1;

 % om1 oxidation by sulfate
 R{9}             = cx1*om1 + 0.5*cx1*so4 + (cy1-2*cz1)*co_2 + (cy1-2*cz1)*h2o - (cx1+cy1-2*cz1)*hco_3 - 0.5*cx1*s2 - cz1*hpo4 - cy1*nh4;
 r{9}             = 'k1_OM * om1 * so4/(so4 + kSO) * kinO/(kinO + o2) * kinNO/(kinNO + no3) * kinMO/(kinMO + mo_1) * kinFOH/(kinFOH + foh) /cx1 ';
 reactionUnits(9) = 1;
 
 % om2 oxidation by sulfate
 R{10}             = cx2*om2 + 0.5*cx2*so4 + (cy2-2*cz2)*co_2 + (cy2-2*cz2)*h2o - (cx2+cy2-2*cz2)*hco_3 - 0.5*cx2*s2 - cz2*hpo4 - cy2*nh4;
 r{10}             = 'k2_OM * om2 * so4/(so4 + kSO) * kinO/(kinO + o2) * kinNO/(kinNO + no3) * kinMO/(kinMO + mo_1) * kinFOH/(kinFOH + foh) / cx2';
 reactionUnits(10) = 1;

 % methanogenesis by om1
 R{11}             = cx1*om1 + (cy1-2*cz1)*h2o - (0.5*cx1-cy1+2*cz1)*co_2 - (cy1-2*cz1)*hco_3 - 0.5*cx1*ch4 - cz1*hpo4 - cy1*nh4;
 r{11}             = 'k1_OM * om1 * (1 - o2/(o2 + kO) - no3/(no3 + kNO) * kinO/(kinO + o2) - mo_1/(mo_1 + kMO) * kinO/(kinO + o2) * kinNO/(kinNO + no3) - foh/(foh + kFOH) * kinO/(kinO + o2) * kinNO/(kinNO + no3) * kinMO/(kinMO + mo_1) - so4/(so4 + kSO) * kinO/(kinO + o2) * kinNO/(kinNO + no3) * kinMO/(kinMO + mo_1) * kinFOH/(kinFOH + foh)) / cx1';
 reactionUnits(11) = 1;
 
 % methanogenesis by om2
 R{12}             = cx2*om2 + (cy2-2*cz2)*h2o - (0.5*cx2-cy2+2*cz2)*co_2 - (cy2-2*cz2)*hco_3 - 0.5*cx2*ch4 - cz2*hpo4 - cy2*nh4;
 r{12}             = 'k2_OM * om2 * (1 - o2/(o2 + kO) - no3/(no3 + kNO) * kinO/(kinO + o2) - mo_1/(mo_1 + kMO) * kinO/(kinO + o2) * kinNO/(kinNO + no3) - foh/(foh + kFOH) * kinO/(kinO + o2) * kinNO/(kinNO + no3) * kinMO/(kinMO + mo_1) - so4/(so4 + kSO) * kinO/(kinO + o2) * kinNO/(kinNO + no3) * kinMO/(kinMO + mo_1) * kinFOH/(kinFOH + foh)) / cx2';
 reactionUnits(12) = 1;
 
 % nitrification
 R{13}             = nh4 + 2*o2 + 2*hco_3 - no3 - 2*co_2 - 3*h2o;
 r{13}             = 'knhox * nh4 * o2 ';
 reactionUnits(13) = 0;
 
 % Mn oxidation by O2
 R{14}             = 2*mn + o2 + 4*hco_3 - 2*mo_2 - 4*co_2 - 2*h2o;
 r{14}             = 'kmox * mn * o2 ';
 reactionUnits(14) = 0;
 
 % Fe oxidation by O2
 R{15}             = fe + 0.25*o2 + 2*hco_3 +0.5*h2o - foh - 2*co_2;
 r{15}             = 'kfox * fe * o2 ';
 reactionUnits(15) = 0;
 
 % H2S oxidation by O2
 R{16}             = s2 + 2*o2 + 2*hco_3 - so4 - 2*co_2 - 2*h2o;
 r{16}             = 'ksox * s2 * o2';
 reactionUnits(16) = 0;

 % Methane oxidation by O2
 R{17}             = ch4 + 2*o2 - co_2 - 2*h2o;
 r{17}             = 'kchox * ch4 * o2 ';
 reactionUnits(17) = 0;

 % Ammonium oxidation by 1st MnO2 pool(mo_1)
 R{18}             = nh4 + 4*mo_1 + 6*h - 4*mn - no3 - 5*h2o;
 r{18}             = 'knhmo * nh4 * mo_1';
 reactionUnits(18) = 1;
 
 % Ammonium oxidation by 2nd MnO2 pool(mo_2)
 R{19}             = nh4 + 4*mo_2 + 6*h - 4*mn - no3 - 5*h2o;
 r{19}             = 'knhmo * nh4 * mo_2 ';
 reactionUnits(19) = 1;

 % Fe oxidation by 1st MnO2 pool(mo_1) 
 R{20}             = 2*fe + mo_1 + 2*hco_3 + 2*h2o - 2*foh - mn - 2*co_2;
 r{20}             = 'kfmo * mo_1 * fe';
 reactionUnits(20) = 1;
 
 % Fe oxidation by 2nd MnO2 pool(mo_2)
 R{21}             = 2*fe + mo_2 + 2*hco_3 + 2*h2o - 2*foh - mn - 2*co_2;
 r{21}             = 'kfmo * mo_2 * fe ';
 reactionUnits(21) = 1;
 
 % H2S oxidation by 1st MnO2 pool(mo_1)
 R{22}             = s2 + 4*mo_1 + 6*co_2 + 2*h2o - 4*mn - so4 - 6*hco_3;
 r{22}             = 'ksmo * mo_1 * s2';
 reactionUnits(22) = 1;
 
 % H2S oxidation by 2nd MnO2 pool(mo_2)
 R{23}             = s2 + 4*mo_2 + 6*co_2 + 2*h2o - 4*mn - so4 - 6*hco_3;
 r{23}             = 'ksmo * mo_2 * s2 ';
 reactionUnits(23) = 1;
 
 % Fe(OH) reduction by H2S
 R{24}             = s2 + 8*foh + 14*co_2 - 8*fe - so4 - 14*hco_3 - 6*h2o;
 r{24}             = 'ksfo * foh * s2';
 reactionUnits(24) = 1;
 
 % vivianite dissolution by H2S
 R{25}             = 3*s2 + vivianite - 3*fs - 2*hpo4 - 4*h;
 r{25}             = 'ksviv * s2 * vivianite';
 reactionUnits(25) = 1;
 
 % anaerobic methane oxidation
 R{26}             = ch4 + so4 + co_2 - 2*hco_3 -s2;
 r{26}             = 'kchso * ch4 * so4';
 reactionUnits(26) = 0;

 % iron sulfide dissolution
 R{27}             = fs + 2*h - fe - s2;
 r{27}             = 'kIronSulfideDis * fs * (1-fe*s2/(kEqIronSulfide*ph^2)) * (fe*s2/(kEqIronSulfide*ph^2)<1)';
 reactionUnits(27) = 1;
 
 % iron sulfide precipitation
 R{28}             = s2 + fe - 2*h - fs;
 r{28}             = 'kIronSulfidePre * (fe*s2/(kEqIronSulfide*ph^2)-1) * (fe*s2/(kEqIronSulfide*ph^2)>=1)';
 reactionUnits(28) = 1;

 % pyrite precipitation
 R{29}             = fs + s2 - pyrite + h2;
 r{29}             = 'kpyrpre * fs * s2';
 reactionUnits(29) = 1;
 
 % vivianite dissolution
 R{30}             = vivianite + 2*h + 8*h2o - 3*fe - 2*hpo4;
 r{30}             = 'kvivdis * vivianite * (1 - (fe^3*(hpo4)^2/kEqViv)^0.2) * (fe^3*(hpo4)^2/kEqViv < 1) * 100';
 reactionUnits(30) = 1;

 % vivianite precipitation
 R{31}             = 3*fe + 2*hpo4 - vivianite - 2*h - 8*h2o;
 r{31}             = 'kvivpre * ((fe^3*(hpo4)^2/kEqViv)^0.2 - 1) * (fe^3*(hpo4)^2/kEqViv>=1)';
 reactionUnits(31) = 1;


 
 
